
/*
 * Copyright 2007 Nuova Systems, Inc.  All rights reserved.
 * Copyright 2009 Cisco Systems, Inc.  All rights reserved.
 */
#ident "$Id: boot.c 21 2016-12-02 21:25:59Z buildsa $"

#include <common.h>
#include <command.h>
#include <flash.h>
#include <asm/byteorder.h>
#include <asm/addrspace.h>
#include <asm/io.h>
#include <common/include/mezzi2c.h>
#include <palo/fw/mcpu/include/palofwmap.h>
#include <palo/fw/mcpu/include/palofwimg.h>
#include <palo/fw/mcpu/include/palopostcodes.h>
#include <palo/include/misc_mregs.h>
#include <palo/include/misc_regs.h>
#include <palo/fw/mcpu/include/board_conf.h>
#include "palodtls.h"

DECLARE_GLOBAL_DATA_PTR;

#if defined(CONFIG_AUTOBOOT_KEYED) && \
    defined(CONFIG_ZERO_BOOTDELAY_CHECK) && \
    defined(CONFIG_AUTOBOOT_STOP_STR)
#define FWBOOT_LATEABORT
#endif

enum {
    FWBOOT_FAILED,
    FWBOOT_ABORTED,
};

int gunzip(void *, int, unsigned char *, unsigned long *);
void invoke_linux(ulong addr, ulong entry, ulong initrd_start, ulong initrd_end);
extern flash_info_t flash_info[];


/*
 * Determine if the manufacuring image has been selected for boot.
 */
static int
is_mfgimage_selected(void)
{
    if (board_conf_has_mfgdiag() && board_conf_mfgdiag() == 0)
        return 1;
    return 0;
}

/*
 * Validate an image header.
 */
static int
is_hdrvalid(fwimghdr_t *hdr, u_int32_t magic)
{
    u_int32_t crc, hcrc, len, addr;
    u_int32_t sdram_len;

    /*
     * Check the header magic and checksum.
     */
    if (magic != 0 && hdr->magic != magic)
        return 0;
    hcrc = hdr->hcrc;
    hdr->hcrc = 0;
    crc = crc32(0, (uchar *)hdr, sizeof(*hdr));
    hdr->hcrc = hcrc;
    if (crc != hcrc)
        return 0;

    len = (hdr->size + 3) & -4; /* round up */
    addr = hdr->load;

    /*
     * Limit the memory size checking to 1/2 the KSEG0 visible capacity,
     * as registers occupy the upper 256MB.
     */
    sdram_len = initdram(0 /* not used */);
    if (sdram_len > 0x10000000) {
        sdram_len = 0x10000000;
    }

    /*
     * Make sure the image loads at an aligned address, does not load on top
     * of U-Boot and it fits in the available memory.
     */
    if ((addr & 0x3) != 0 ||
        addr < CFG_UBOOT_RAMTOP ||
        len > sdram_len ||
        (addr + len) >= (CFG_SDRAM_BASE + sdram_len))
        return 0;

    /*
     * Make sure compressed images fit into the staging area.
     */
    if ((hdr->flags & PALOFWIMGF_GZIPPED) &&
        ((CFG_GZIPIMG_BASE + len) >= (CFG_SDRAM_BASE + sdram_len))) {
        return 0;
    }
        
    return 1;
}

/*
 * Load an image with a pre-validated header.
 * hdr = image header (in ram).
 * data = address of image data in flash (past the header).
 *
 * Return the entry point.
 */
static u_int32_t
fwload(fwimghdr_t *hdr, u_int32_t data)
{
    int doverify, dounzip, maxlen;
    u_int32_t cpdest, load;
    unsigned long len;
    char *s;

    s = getenv("verify");
    doverify = (s && (*s == 'n')) ? 0 : 1;

    dounzip = ((hdr->flags & PALOFWIMGF_GZIPPED) != 0);
    load = hdr->load;
    len = hdr->size;

    cpdest = dounzip ? CFG_GZIPIMG_BASE : load;
    printf("Copying %simage to RAM [0x%08x, +%08x => 0x%08x]...",
            dounzip ? "GZIP " : "", data, len, cpdest);
    blockcopy((void *)cpdest, (void *)data, len);

    if (doverify) {
        puts("\nVerifying Checksum ...");
        if (crc32(0, (uchar *)cpdest, len) != hdr->dcrc) {
            puts("\nBad Data CRC\n");
            return 0;
        }
    }

    if (dounzip) {
        printf("\nUncompressing...");
        maxlen = CFG_GZIPIMG_BASE - load;
        if (gunzip((void *)load, maxlen, (uchar *)cpdest, &len) != 0) {
            puts("\nGUNZIP ERROR\n");
            return 0;
        }
    }
    puts("\nOK\n");

    flush_cache(load, len); /* really flushes entire dcache */

    return hdr->ep;
}


static void
strsanitize(char *s)
{
    char c;

    while ((c = *s) != '\0') {
        if (c < ' ' || c >= 127)
            *s = '.';
        ++s;
    }
}


#ifdef FWBOOT_LATEABORT
/*
 * Offer a late attempt to stop boot.
 */
static int
late_abortboot(void)
{
    char *abortstr = CONFIG_AUTOBOOT_STOP_STR;
    int i;

    i = 0;
    while (tstc() && abortstr[i] != '\0') {
        i = (getc() == abortstr[i]) ? (i + 1) : 0;
    }
    return (abortstr[i] == '\0') ? 0 : -1;
}
#endif


/*
 * Boot an image at the supplied address.
 */
static int
fwboot(u_int32_t addr, u_int32_t magic)
{
    extern const char paloswvers[];
    fwimghdr_t hdr;
    u_int32_t entry;
    char buf[32];

    memcpy(&hdr, (void *)addr, sizeof(hdr));
    if (!is_hdrvalid(&hdr, magic)) {
        printf("Invalid image header\n");
        return FWBOOT_FAILED;
    }

    entry = fwload(&hdr, addr + sizeof(hdr));
    if (entry == 0)
        return FWBOOT_FAILED;

#ifdef FWBOOT_LATEABORT
    if (late_abortboot() == 0) {
        puts("\n");
        return FWBOOT_ABORTED;
    }
#endif

    sprintf(buf, "0x%x", addr);
    setenv("fwaddr", buf);
    strsanitize((char *)hdr.build);
    setenv("fwbuild", (char *)hdr.build);
    setenv("fwname", "Palo Firmware"); // compatibility
    strsanitize((char *)hdr.vers);
    setenv("fwvers", (char *)hdr.vers);
    setenv("bootfwvers", (char *)paloswvers);

    invoke_linux(addr, entry, 0, 0);

    return FWBOOT_FAILED;
}



/*
 * Retrieve the active directory from a firwmare map.
 */
static fwdir_t *
fwmap_get_active_dir(fwmap_t *mp)
{
    int i;

    /*
     * The active one is the last entry with a good magic.
     */
    for (i = PALOFW_NDIRS - 1; i >= 0; i--) {
        if (mp->dir[i].magic == PALOFW_DIRMAGIC)
            return &mp->dir[i];
    }
    return NULL;
}

/*
 * Find the active firmware map.
 */
static fwmap_t *
get_active_fwmap(void)
{
    fwmap_t *maps[] = {
        (fwmap_t *)palo_part_addr(PT_FWMAP),
        (fwmap_t *)(palo_part_addr(PT_FWMAP) + palo_part_size(PT_FWMAP) / 2)
    };
    u_int32_t bestgeneration = 0;
    fwmap_t *map, *bestmap;
    int i;

    bestmap = NULL;
    for (i = 0; i < sizeof(maps) / sizeof(maps[0]); i++) {
        map = maps[i];
        if (map->magic != PALOFW_MAPMAGIC)
            continue;
        if (bestmap == NULL || (int)(map->generation - bestgeneration) > 0) {
            bestmap = map;
            bestgeneration = map->generation;
        }
    }
    return bestmap;
}

/*
 * Retrieve the active directory.
 */
static fwdir_t *
get_fwdir(void)
{
    fwdir_t *dp = NULL;
    fwmap_t *mp;

    mp = get_active_fwmap();
    if (mp != NULL)
        return fwmap_get_active_dir(mp);

    return dp;
}

/*
 * Retrieve a validated directory entry.
 */
static fwdirent_t *
get_fwdirent(fwdir_t *dp, int idx)
{
    fwdirent_t *de;
    fwimghdr_t *hdr;
    fwimghdr_t xhdr;

    de = &dp->dirent[idx];
    if (de->magic != PALOFW_DIRENTMAGIC || de->naddr != ~de->addr)
	return NULL;
    hdr = (fwimghdr_t *)de->addr;
    if (hdr->magic != PALOFW_IMGMAGIC)
        return NULL;
    memcpy(&xhdr, hdr, sizeof(xhdr));
    return (is_hdrvalid(&xhdr, PALOFW_IMGMAGIC)) ? de : NULL;
}
                                    


/*
 * Search the entire flash for an image, as a last resort, in the absense
 * of a firmware map.
 */
static u_int32_t
find_fwimage_nomap(int idx)
{
    flash_info_t *info = &flash_info[0];
    fwimghdr_t header;
    fwimghdr_t *hdr;
    u_int32_t addr;
    int i;

    if (info->flash_id == FLASH_UNKNOWN)
        return 0;
    for (i = 0; i < info->sector_count; i++) {
        addr = info->start[i];
        hdr = (fwimghdr_t *)addr;
        if (hdr->magic == PALOFW_IMGMAGIC) {
            memcpy(&header, hdr, sizeof(header));
            if (is_hdrvalid(&header, PALOFW_IMGMAGIC) && idx-- == 0)
                return addr;
        }
    }

    return 0;
}


/*
 * fwboot command for PALO.
 */
static int
do_fwboot(cmd_tbl_t *cmdtp, int flag, int argc, char *argv[])
{
    flash_info_t *info = &flash_info[0];
    int i, is_first, pass, diag_tried;
    u_int32_t addr, bcmd;
    fwdirent_t *de;
    char buf[15];
    fwdir_t *dp;

    if (argc > 1) {
        addr = simple_strtoul(argv[1], NULL, 16);
        if (addr >= info->start[0] && addr < info->start[0] + info->size) {
            /* Address */
            if (addr & 0x3) {
                printf("Invalid address\n");
                return 1;
            }
        } else {
            /* Id */
            dp = get_fwdir();
            for (i = 0; i < PALOFW_NDIRENTS; i++) {
                de = get_fwdirent(dp, i);
                if (de == NULL)
                    continue;
                if (--addr == 0) {
                    addr = de->addr;
                    break;
                }
            }
            if (i == PALOFW_NDIRENTS) {
                printf("Invalid index\n");
                return 1;
            }
        }
        write_post(PALO_POST_START_APP_FIRMWARE);
        fwboot(addr, 0);
        return 1;
    }

    /*
     * PALO Firmware Auto-boot.
     */

    /*
     * Read I2C boot control commands.
     */
    bcmd = 0;
    if (read_bcmd(&bcmd) == 0 && bcmd != 0) {
        printf("## Boot options = 0x%x\n", bcmd);
        sprintf(buf, "0x%x", bcmd);
        setenv("bootopt", buf);
        if (bcmd & BOOTCMD_LOAD_ALTAPP)
            setenv("load_altapp", "1");
        if (bcmd & BOOTCMD_LOAD_DIAG)
            setenv("load_diag", "1");
        if (bcmd & BOOTCMD_OPT_DIAGMODE)
            setenv("diagmode", "1");
        if (bcmd & BOOTCMD_OPT_NOVNICS)
            setenv("ignore_vniccfg", "1");
        if (bcmd & BOOTCMD_OPT_NOCONFIG)
            setenv("ignore_palocfg", "1");
        if (bcmd & BOOTCMD_OPT_NOPOST)
            setenv("skip_post", "1");
        if (bcmd & BOOTCMD_OPT_DEBUGVNIC)
            setenv("want_debugvnic", "1");
    }

    /*
     * Set power and pcie mode options.
     */
    if (board_conf_is_lowpower()) {
        setenv("lowpower", "1");
        setenv("pcieoff", "1");
    }

    /*
     * Check for manufacturing diags selector.
     */
    diag_tried = 0;
    if ((bcmd & BOOTCMD_LOAD_DIAG) || is_mfgimage_selected()) {
        printf("## Booting manufacturing firmware ...\n");
        addr = palo_part_addr(PT_DIAG);
        diag_tried = 1;
        write_post(PALO_POST_START_DIAG_FIRMWARE);
        if (fwboot(addr, PALOFW_DIAGMAGIC) == FWBOOT_ABORTED)
            return 1;
        printf("## Failed.  Reverting to production firmware\n");
    }

    /*
     * Search for production firmware.
     * Max two passes - if the LOAD_ALTAPP boot option is set then
     * the first time around we skip the the first (active) image.
     */
    dp = get_fwdir();
    if (dp != NULL) {
        for (pass = 0; pass < ((bcmd & BOOTCMD_LOAD_ALTAPP) ? 2 : 1); pass++) {
            is_first = 1;
            for (i = 0; i < PALOFW_NDIRENTS; i++) {
                de = get_fwdirent(dp, i);
                if (de == NULL)
                    continue;
                if (pass == 1 || !is_first || !(bcmd & BOOTCMD_LOAD_ALTAPP)) {
                    printf("## Booting image at %08lx ...\n", de->addr);
                    write_post(PALO_POST_START_APP_FIRMWARE);
                    if (fwboot(de->addr, PALOFW_IMGMAGIC) == FWBOOT_ABORTED)
                        return 1;
                }
                is_first = 0;
            }
        }
    }

    /*
     * No firmware found in the directory.  Try the diag image if we have
     * not already tried it.
     */
    if (!diag_tried) {
        printf("No firmware found in directory - trying diag firmware...\n");
        addr = palo_part_addr(PT_DIAG);
        write_post(PALO_POST_START_DIAG_FIRMWARE);
        if (fwboot(addr, PALOFW_DIAGMAGIC) == FWBOOT_ABORTED)
            return 1;
    }

    /*
     * Diag firmware not available.  Loop over the flash trying to
     * boot anything that looks like an image.
     */
    printf("No firmware found in directory - searching...\n");
    i = 0;
    for (i = 0;; i++) {
        addr = find_fwimage_nomap(i);
        if (addr == 0)
            break;
        printf("## Booting image at %08lx ...\n", addr);
        write_post(PALO_POST_START_APP_FIRMWARE);
        if (fwboot(addr, PALOFW_IMGMAGIC) == FWBOOT_ABORTED)
            return 1;
    }

    write_post(PALO_POST_HALT_NO_FIRMWARE);

    printf("## panic: No firmware\n");

    /*
     * Spin an die, but allow a host reset to restart us.
     */
#if defined CONFIG_BASE_PALO || defined CONFIG_BASE_SERENO
    writel(1, KSEG1ADDR(MISC_MREGS_PCIE_FORCE_RESET_OK));
#endif // CONFIG_BASE
    for (;;) {
        asm volatile(" wait");
    }

    return 1;
}

U_BOOT_CMD(
    fwboot, CFG_MAXARGS, 1, do_fwboot,
    "fwboot  - boot palo firmware image from memory\n",
    "[(addr|id) [arg ...]]\n    - boot application image stored in memory\n"
    "\tpassing arguments 'arg ...'; when booting a Linux kernel,\n"
    "\t'arg' can be the address of an initrd image\n"
);



static int
do_fwls(cmd_tbl_t *cmdtp, int flag, int argc, char *argv[])
{
    char buf[128];
    fwimghdr_t *hdr;
    fwdirent_t *de;
    int i, id, len;
    fwdir_t *dp;

    dp = get_fwdir();
    if (dp == NULL) {
        printf("No directory\n");
        return 1;
    }

    id = 1;
    for (i = 0; i < PALOFW_NDIRENTS; i++) {
        de = get_fwdirent(dp, i);
        if (de == NULL)
            continue;
        hdr = (fwimghdr_t *)de->addr;
        len = min(sizeof(buf), sizeof(hdr->vers));
        memcpy(buf, hdr->vers, len);
        buf[len - 1] = '\0';
        strsanitize(buf);
        printf("%2d. [%08x] %s", id++, de->addr, buf);

        len = min(sizeof(buf), sizeof(hdr->build));
        memcpy(buf, hdr->build, len);
        buf[len - 1] = '\0';
        strsanitize(buf);
        printf(", %s\n", buf);
    }

    return 0;
}

U_BOOT_CMD(
    fwls, 1, 1, do_fwls,
    "fwls    - list available firmware images\n",
    NULL
);

static int
do_bootopt(cmd_tbl_t *cmdtp, int flag, int argc, char *argv[])
{
    u_int32_t val;

    if (argc == 1) {
        if (read_bcmd(&val) < 0)
            puts("% Failed\n");
        else
            printf("0x%x\n", val);
    } else {
        val = simple_strtoul(argv[1], NULL, 0);
        if (write_bcmd(val) < 0)
            puts("% Failed\n");
    }
    return 0;
}

U_BOOT_CMD(
    bootopt, CFG_MAXARGS, 1, do_bootopt,
    "bootopt  - show/set i2c boot options\n",
    "[val]\n    - value to write\n"
);

extern void booti2c_write_post_code(u_int32_t val, u_int32_t cardid);

static int
do_bootpost(cmd_tbl_t *cmdtp, int flag, int argc, char *argv[])
{
    u_int32_t val;
    u_int32_t regs[13];

    if (argc < 2) {
        printf("Args required\n");
        return 1;
    }
    val = simple_strtoul(argv[1], NULL, 0);

    asm volatile("      sw $16, %0" : "=m"(regs[0]));
    asm volatile("      sw $17, %0" : "=m"(regs[1]));
    asm volatile("      sw $18, %0" : "=m"(regs[2]));
    asm volatile("      sw $19, %0" : "=m"(regs[3]));
    asm volatile("      sw $20, %0" : "=m"(regs[4]));
    asm volatile("      sw $21, %0" : "=m"(regs[5]));
    asm volatile("      sw $22, %0" : "=m"(regs[6]));
    asm volatile("      sw $23, %0" : "=m"(regs[7]));
    asm volatile("      sw $24, %0" : "=m"(regs[8]));
    asm volatile("      sw $25, %0" : "=m"(regs[9]));
    asm volatile("      sw $26, %0" : "=m"(regs[10]));
    asm volatile("      sw $27, %0" : "=m"(regs[11]));

    booti2c_write_post_code(val, board_conf_card_address());

    asm volatile("      lw $16, %0" : : "m"(regs[0]));
    asm volatile("      lw $17, %0" : : "m"(regs[1]));
    asm volatile("      lw $18, %0" : : "m"(regs[2]));
    asm volatile("      lw $19, %0" : : "m"(regs[3]));
    asm volatile("      lw $20, %0" : : "m"(regs[4]));
    asm volatile("      lw $21, %0" : : "m"(regs[5]));
    asm volatile("      lw $22, %0" : : "m"(regs[6]));
    asm volatile("      lw $23, %0" : : "m"(regs[7]));
    asm volatile("      lw $24, %0" : : "m"(regs[8]));
    asm volatile("      lw $25, %0" : : "m"(regs[9]));
    asm volatile("      lw $26, %0" : : "m"(regs[10]));
    asm volatile("      lw $27, %0" : : "m"(regs[11]));

    return 0;
}


U_BOOT_CMD(
    bootpost, CFG_MAXARGS, 2, do_bootpost,
    "bootpost  - write post using boot code\n",
    "val\n - value to write\n"
);

static int
do_showpost(cmd_tbl_t *cmdtp, int flag, int argc, char *argv[])
{
    u_int32_t postval;

    if (read_post(&postval) == 0)
        printf("0x%x\n", postval);

    return 0;
}

U_BOOT_CMD(
    showpost, CFG_MAXARGS, 1, do_showpost,
    "showpost  - show post code\n",
    NULL
);
