
/*
 * Copyright (c) 2010 Cisco Systems, Inc.  All rights reserved.
 */
#ident "$Id: findvar.S 21 2016-12-02 21:25:59Z buildsa $"

#include <config.h>
#include <version.h>
#include <palo/fw/mcpu/include/paloflash.h>
#include <palo/fw/mcpu/include/ubootenv.h>
#include <asm/regdef.h>
#include <asm/mipsregs.h>
#include <asm/addrspace.h>
#include "palodtls.h"

/*
 * Find an environment variable.
 * Derived from: mips_4KCle-gcc -Os -S
 *
 * char *
 * findvar(char *var)
 * {
 *     char *p, *q;
 *     char *e;
 * 
 *     p = (char *)CFG_ENV_ADDR + 4;
 *     e = (char *)CFG_ENV_ADDR + CFG_ENV_SIZE;
 *     while (p < e && *p != '\0') {
 *         q = var;
 *         while (q < e && *p++ == *q++)
 *             ;
 *         if (*(q - 1) == '\0')
 *             return p - 1;
 *         while (p < e && *p != '\0')
 *             ++p;
 *         ++p;
 *     }
 *     
 *     return NULL;
 * }
 *
 * findvar() is available very early on - before RAM initialization and
 * before the flash type is known.  As the environment may be located in
 * two places (dependent on flash type), we need to probe for the location.
 * crc32() is unavailable so the following assumptions are used:
 *      1) The environment data will be at least 64 bytes.
 *      2) The character set is 7-bit ASCII.
 *      3) As our usage is light, the end of the area will be full of zeroes.
 *
 * The probe algorithm is as follows:
 *   Bytes 0004-0043 (first 64 bytes of data) must sum non-zero w/ bit 7
 *   unset in all bytes.
 *   Bytes 77C0-7FFF (last 64 bytes) must be zero.
 *
 * We test the P30 area first, as that will fail quickly for M29 flash (that
 * area is erased (all 0xFFs)).
 *
 * fv_probe takes the probe address in t0 and returns the same value if the
 * test passes, or zero if not.
 */

        .set mips32r2
        .set noreorder
        .text

#define OFFS_TO_VADDR(x)        KSEG1ADDR(PALOFLASH_PHYS + (x))

/*
 * In:
 *  t0 = pointer to environment to test.
 * Out:
 *  t0 - unchanged if test passes; zero if test fails.
 * Clobber:
 *  a1-a3, v0-v1, t0-t1
 */

#define LOW_PTR     a1
#define HIGH_PTR    a2
#define BIT_7_MASK  a3
#define NWORDS      v0
#define VAL         v1

        .ent    fv_probe
fv_probe:
        addiu   LOW_PTR, t0, 4          // low_ptr = &env->data[0];
        li      t1, CFG_ENV_SIZE - 64   // high_ptr = &env->data[xx-64]
        addu    HIGH_PTR, t0, t1
        li      BIT_7_MASK, 0x80808080  // check bit-7 in all bytes
        li      NWORDS, (64 / 4) - 1    // nwords = 15
1:
        lw      VAL, 0(LOW_PTR)         // val = *low_ptr
        addiu   LOW_PTR, 4              // low_ptr += 4
        beqz    VAL, 2f                 // fail if val == 0
        nop
        and     VAL, BIT_7_MASK         // val &= 0x80808080
        bnez    VAL, 2f                 // fail if any bit-7 set
        nop
        lw      VAL, 0(HIGH_PTR)        // val = *high_ptr
        addiu   HIGH_PTR, 4             // high_ptr += 4
        bnez    VAL, 2f                 // fail if non-zero
        nop
        bnez    NWORDS, 1b              // loop nwords
        addiu   NWORDS, -1              // --nwords
        jr      ra                      // success
        nop                             // returning t0 unchanged
2:
        jr      ra                      // fail
        move    t0, zero                // returning t0 = 0
        .end    fv_probe

/*
 * In:
 *  a0 = search string (must end with '=', e.g. "fastboot="
 *
 * Out:
 *  v0 = 0, or pointer to found env character following '='
 */
#define VAR     a0
#define P       v0
#define Q       v1
#define E       a1
#define CP      a2
#define CQ      a3

LEAF(findvar)
        move    t2, ra                  // save ra
        li      t0, OFFS_TO_VADDR(PALOFLASH_P30_ENV_OFFSET)
        BAL     (fv_probe)
        nop
        bnez    t0, $found_env          // branch if t0 != 0 (good case)
        nop
        li      t0, OFFS_TO_VADDR(PALOFLASH_M29_ENV_OFFSET)
        BAL     (fv_probe)
        nop
        beqz    t0, $at_eof             // branch to EOF if t0 == 0 (bad case)
        move    ra, t2                  // (delay slot) restore ra

$found_env:
        move    ra, t2                  // restore ra
        li      t1, CFG_ENV_SIZE - 1    // E = &env->data[CFG_ENV_SIZE - 1];
        addu    E, t0, t1
        addiu   P, t0, 4                // P = &env->data[0]
        b       $begin
        nop

$start_word:
1:
        sltu    t0, E, P                // check end < tmp_p
        bnez    t0, $at_eof             // bail if so
        nop
        lb      CQ, 0(Q)                // cq = *q
        addiu   Q, Q, 1                 // q++
        lb      CP, 0(P)                // cp = *p
        beq     CQ, CP, 1b              // loop until (cq != cp)
        addiu   P, P, 1                 // with p++

        beqz    CQ, $done               // if (cq == '\0') goto done
        addiu   P, P, -1                // with --p

1:
        sltu    t0, E, P                // check end < p
        bnez    t0, $at_eof             // bail if so
        nop
        lb      CP, 0(P)                // cp = *p
        bnez    CP, 1b                  // loop until end of word
        addiu   P, P, 1                 // with p++

        sltu    t0, E, P                // check end < p
        bnez    t0, $at_eof             // bail if so
        nop

$begin:
        lb      CP, 0(P)                // cp = *p
        bnez    CP, $start_word         // if (cp != 0) goto start_word
        move    Q, VAR                  // with q = var

$at_eof:
        move    P, zero                 // p = NULL
$done:
        j       ra                      // return (v0 = p)
        nop
        END(findvar)
