
/*
 * Copyright 2013 Cisco Systems, Inc.  All rights reserved.
 */
#ident "$Id: lowlevel_boardid.S 21 2016-12-02 21:25:59Z buildsa $"

#include <config.h>
#include <version.h>
#include <asm/regdef.h>
#include <asm/mipsregs.h>
#include <asm/addrspace.h>
#include <palo/include/palo_gpio.h>
#include <palo/include/synopsis/dw_gpio.h>
#include <palo/fw/mcpu/include/paloflash.h>
#include <palo/fw/mcpu/include/board_conf.h>
#include <palo/fw/mcpu/include/boardidrec.h>
#include "palodtls.h"

#define BOARDID_PHYS    (PALOFLASH_PHYS + PALOFLASH_M29_BOARDID_OFFSET)

#if defined(CONFIG_BASE_SERENO)
#define DEFAULT_BOARD   BOARD_CONF_BRD_GREENWOOD
#elif defined(CONFIG_BASE_CRUZ)
#define DEFAULT_BOARD   BOARD_CONF_BRD_CALIMESA
#endif

        .set mips32r2
        .set noreorder
        .text

/*
 * lowlevel_boardid - Obtain the BOARD_CONF_BRD_xxx value for the board.
 * This function is intended to be called from low-level boot code prior
 * to the availability of RAM and a stack frame (at which point the real
 * board_conf_xxx are available.
 *
 * The #ifdef CONFIG_BASE_xxx fun below results in three algorithms
 * depending on the ASIC type:
 *      Palo:       id = map[GPIO]; return id;
 *      Sereno:     id = map[GPIO]; if (id < 0) { id = FLASH.id } return id;
 *      Cruz:       id = FLASH.id; return id;
 *
 * Palo boards never used the FLASH to obtain their id.
 * Older Sereno boards are identified by GPIO, but newer ones use a common
 * GPIO id and their true board id comes from FLASH.
 * Cruz boards get their id from FLASH (except CALIMESA boards that were
 * never programmed, so that's the default).
 *
 * The clobber list is explicit here so callers may use this function in
 * contexts where saving registers may be awkward.
 *
 * Output:      v0 = board id
 * Clobbers:    t0-t2
 */
LEAF(lowlevel_boardid)
#if defined(CONFIG_BASE_PALO) || defined(CONFIG_BASE_SERENO)
        /*
         * Palo and Sereno consult the GPIOs to determine their board type.
         * The pins used are common.
         * We read the two GPIO external data registers, mask the bits we
         * want, OR the two together and shift the result into position
         * to leave the GPIO board type [0..7] in t0.
         */
        lw      t0, KSEG1ADDR(PL_GPIO_EXT(PL_BL_GPIO_BOARDTYPE_0))  // xx0x
        lw      t1, KSEG1ADDR(PL_GPIO_EXT(PL_BL_GPIO_BOARDTYPE_2))  // 21xx
        andi    t0, 0x2                                             // __0_
        andi    t1, 0xc                                             // 21__
        or      t0, t1                                              // 210_
        srl     t0, 1                                               // _210

        /*
         * Use an 8-entry lookup table to decode the id.
         * t0 is used as the index into gpio_brd_tab[], a signed byte array
         * which maps from GPIO value to BOARD_CONF_BRD_xxx value.
         */
        la      t1, gpio_brd_tab                    // gpio to boardid table
        addu    t1, t0                              // byte index using gpio
        lb      v0, 0(t1)                           // v0 = BOARD_CONF_BRD_xxx
#if defined(CONFIG_BASE_SERENO)
        /*
         * Sereno boards may need to reference flash, in which case the
         * table returns a negative value.
         */
        bgez    v0, 1f
        nop
#endif /* CONFIG_BASE_SERENO */
#endif /* CONFIG_BASE_PALO || CONFIG_BASE_SERENO */
#if defined(CONFIG_BASE_SERENO) || defined(CONFIG_BASE_CRUZ)
        /*
         * Sereno and Cruz consult the boardidrec_t in flash.
         * This structure has three fields: magic, boardid, sum.  When
         * all three words are added together the result should be zero.
         * If either the magic or sum check fails we return the default
         * board id for the ASIC type.
         */
        li      t0, KSEG1ADDR(BOARDID_PHYS)         // t0 = &boardidrec_t
        lw      t1, BOARDIDREC_MAGIC_OFFSET(t0)     // t1 = t0->magic
        li      t2, BOARDIDREC_MAGIC                // t2 = MAGIC
        bne     t1, t2, 2f                          // branch if no match
        nop
        lw      v0, BOARDIDREC_ID_OFFSET(t0)        // v0 = t0->brdid
        lw      t2, BOARDIDREC_SUM_OFFSET(t0)       // t2 = t0->sum
        addu    t1, v0                              // t1 += brdid
        addu    t1, t2                              // t1 += sum
        beqz    t1, 1f                              // branch if sum is good
        nop
2:
        li      v0, DEFAULT_BOARD                   // default return
1:
#endif /* CONFIG_BASE_SERENO || CONFIG_BASE_CRUZ */
        jr      ra
        nop
        END(lowlevel_boardid)

/*
 * GPIO board type to BOARD_CONF_BRD_xxx map
 */
#if defined(CONFIG_BASE_PALO) || defined(CONFIG_BASE_SERENO)
gpio_brd_tab:
#if defined(CONFIG_BASE_PALO)
        .byte   BOARD_CONF_BRD_BAYLANDS         /* 000 */
        .byte   BOARD_CONF_BRD_PALOALTO         /* xx1 */
        .byte   BOARD_CONF_BRD_MONTEREYPARK     /* 010 */
        .byte   BOARD_CONF_BRD_PALOALTO         /* xx1 */
        .byte   BOARD_CONF_BRD_BAYLANDS         /* default */
        .byte   BOARD_CONF_BRD_PALOALTO         /* xx1 */
        .byte   BOARD_CONF_BRD_BAYLANDS         /* default */
        .byte   BOARD_CONF_BRD_PALOALTO         /* xx1 */
#elif defined(CONFIG_BASE_SERENO)
        .byte   BOARD_CONF_BRD_GREENWOOD        /* 000 */
        .byte   BOARD_CONF_BRD_VASONA           /* 001 */
        .byte   BOARD_CONF_BRD_LEXINGTON        /* 010 */
        .byte   BOARD_CONF_BRD_KIRKWOODLAKE     /* 011 */
        .byte   -1                              /* 100 - use flash */
        .byte   BOARD_CONF_BRD_COTATI           /* 101 */
        .byte   BOARD_CONF_BRD_OAKLAND          /* 110 */
        .byte   BOARD_CONF_BRD_ICEHOUSE         /* 111 */
#endif /* CONFIG_BASE_SERENO */
#endif /* CONFIG_BASE_PALO || CONFIG_BASE_SERENO */

/*
 * lowlevel_is_<name> functions returns true if running on the board named.
 * The clobber list is explicit here so callers may use this function in
 * contexts where saving registers may be awkward.
 *
 * Output:      v0 = non-zero if running on board <name>
 * Clobbers:    a0, t0-t3
 */
#if defined CONFIG_BASE_PALO

LEAF(lowlevel_is_montereypark)
        b       lowlevel_is_xxx                     // clobbers t0-t3
        li      a0, BOARD_CONF_BRD_MONTEREYPARK     // delay slot
        END(lowlevel_is_montereypark)

#elif defined CONFIG_BASE_CRUZ

LEAF(lowlevel_is_montclair)
        b       lowlevel_is_xxx                     // clobbers t0-t3
        li      a0, BOARD_CONF_BRD_MONTCLAIR        // delay slot
        END(lowlevel_is_montclair)

LEAF(lowlevel_is_calistoga)
        b       lowlevel_is_xxx                     // clobbers t0-t3
        li      a0, BOARD_CONF_BRD_CALISTOGA        // delay slot
        END(lowlevel_is_calistoga)

LEAF(lowlevel_is_mountainview)
        b       lowlevel_is_xxx                     // clobbers t0-t3
        li      a0, BOARD_CONF_BRD_MOUNTAINVIEW     // delay slot
        END(lowlevel_is_mountainview)
#endif

/*
 * lowlevel_is_xxx helper.
 * Input:       a0 = desired board id
 * Output:      v0 = non-zero if running on board a0
 * Clobbers:    t0-t3
 */
        .ent    lowlevel_is_xxx
lowlevel_is_xxx:
        move    t3, ra
        BAL     (lowlevel_boardid)                  // clobbers t0-t2
        nop
        xor     v0, a0
        jr      t3
        sltiu   v0, 1                               // delay slot
        .end    lowlevel_is_xxx
