
/*
 * Copyright 2007 Nuova Systems, Inc.  All rights reserved.
 * Copyright 2009 Cisco Systems, Inc.  All rights reserved.
 */
#ident "$Id: palo.c 6 2016-12-02 03:26:55Z buildsa $"

#include <common.h>
#include <command.h>
#include <asm/addrspace.h>
#include <asm/io.h>
#include <palo/include/misc_regs.h>
#include <palo/include/intrc_mregs.h>
#include <palo/include/palo_impl.h>
#include <palo/fw/mcpu/include/board_conf.h>
#include "palodtls.h"

DECLARE_GLOBAL_DATA_PTR;

int
palo_get_board_type(void)
{
    return readl(KSEG1ADDR(MISC_MREGS_EMULATION));
}

/*
 * This function determines the counter frequency, which is
 * half of the CPU clock frequency.
 */
unsigned long
palo_get_cfghz(void)
{
    if (gd->cfg_hz == 0) {
        switch (palo_get_board_type()) {
#ifdef MISC_EMULATION_SIM
        case MISC_EMULATION_SIM:
            gd->cfg_hz = PALO_SIM_CPU_CLK;
            break;
#endif /* MISC_EMULATION_SIM */

        case MISC_EMULATION_ASIC:
            gd->cfg_hz = PALO_ASIC_CPU_CLK;
            break;

        default:
            gd->cfg_hz = PALO_FPGA_CPU_CLK;
            break;
        }
        if (gd->cfg_hz >= 1000000)
            printf("CPU Clock: %uMHz\n", gd->cfg_hz / 1000000);
        else
            printf("CPU Clock: %ukHz\n", gd->cfg_hz / 1000);
    }

    return gd->cfg_hz / 2;
}

#if defined CONFIG_BASE_SERENO

/*
 * Need about a microsecond delay. A longer delay doesn't affect
 * functionality at all. udelay() doesn't work here, nor does
 * spinwait_ns from i2c.
 *
 * Do something in the loop that shouldn't get optimzed away.
 */
static int
usec_delay(void)
{
    int i;
    int val = 0;

    for (i = 0; i < 500; i++) {
        val++;
    }
    return val;
}

/*
 * Sereno B can use the ASIC core clock for the UART.
 * See also similar version of this in
 * sa/src/palo/fw/mcpu/lib/libmcpu/src/palo_conf.c
 */
static void
palo_use_core_clk_for_uart(void)
{
    u_int32_t val;

    writereg(MISC_MREGS_UART_CLK_SEL, 1);

    val = readreg(MISC_MREGS_SW_HARD_RST);
    val |= MISC_MREGS_SW_HARD_RST_APB | MISC_MREGS_SW_HARD_RST_UART;
    writereg(MISC_MREGS_SW_HARD_RST, val);
    usec_delay();  // wait for new clock signal to propagate
    val &= ~(MISC_MREGS_SW_HARD_RST_APB | MISC_MREGS_SW_HARD_RST_UART);
    writereg(MISC_MREGS_SW_HARD_RST, val);
    usec_delay();  // wait for reset signal to exit expander
}

#endif /* CONFIG_BASE */

/*
 * This function determines the UART clock frequency.
 */
unsigned long
palo_get_uartclk(void)
{
    switch (palo_get_board_type()) {
#if defined MISC_EMULATION_SIM
    case MISC_EMULATION_SIM:
        return PALO_SIM_UART_CLK;
#endif // MISC_EMULATION_SIM
    case MISC_EMULATION_ASIC:
#if defined CONFIG_BASE_SERENO
        if (readreg(MISC_MREGS_VERSION) > 0xa0) {
            palo_use_core_clk_for_uart();
            return PALO_ASIC_CORE_CLK;
        }
#endif /* CONFIG_BASE */
        return PALO_ASIC_UART_CLK;
    case MISC_EMULATION_FPGA:
    default:
        return PALO_FPGA_UART_CLK;
    }
}

long int
initdram(int board_type)
{
    switch (palo_get_board_type()) {
    case MISC_EMULATION_ASIC:
        return CFG_SDRAM_LEN_ASIC;
    case MISC_EMULATION_FPGA:
        return CFG_SDRAM_LEN_FPGA;
    default:
        printf("unknown palo board_type %d\n", palo_get_board_type());
        return CFG_SDRAM_LEN_FPGA;
    }
}

int
checkboard(void)
{
    return 0;
}

int
misc_init_r(void)
{
    board_conf_init();
    return 0;
}

void
palo_reset(void)
{
#ifdef CONFIG_BASE_PALO
    if (palo_get_board_type() == MISC_EMULATION_SIM) {
        asm volatile(
            "   .set mips32r2\n"
            "   mfc0 $2, $12,0\n"
            "   li  $3, 0xf0c7ffff\n"
            "   and $2, $2, $3\n"
            "   li  $3, 0x00400004\n"
            "   or  $2, $2, $3\n"
            "   mtc0 $2, $12,0\n"
            "   ehb\n"
            "\n"
            "   mfc0 $2, $16,0\n"
            "   li  $3, 0xfffffffa\n"
            "   and $2, $2, $3\n"
            "   li  $3, 0x00040002\n"
            "   or  $2, $2, $3\n"
            "   mtc0 $2, $16,0\n"
            "   li  $2, 0xbfc00000\n"
            "   jr.hb $2\n"
            "   nop\n" : : );
        for (;;);
    }
#endif /* CONFIG_BASE_PALO */
    writel(1, KSEG1ADDR(INTRC_MREGS_RESET_CPU0));
    /* NOT REACHED */
    for (;;);
}
