
/*
 * Copyright 2008 Cisco Systems, Inc.  All rights reserved.
 */
#ident "$Id$"

#include <common.h>
#include <command.h>
#include <asm/addrspace.h>
#include <asm/io.h>
#include <palo/include/misc_mregs.h>
#include <palo/include/synopsis/dw_i2c.h>
#include <palo/include/palo_i2ctiming.h>
#include "palo_i2c.h"
#include "palodtls.h"

static struct palo_i2c gi2c;

static int
is_timedout(struct palo_i2c *i2c)
{
    if ((int)((sa_timer_get() - i2c->xfer_timeout)) > 0)
        return (1);
    else
        return (0);
}

static int
palo_i2c_master_initialize(struct palo_i2c *i2c, u_int16_t target)
{
    int port = i2c->port;

    /*
     * Ensure that we don't disable the I2C controller mid-operation.
     */
    while (!is_timedout(i2c)) {
        if (!(readreg(IC_STATUS(port)) & IC_STATUS_MST_ACTIVITY)) {
            break;
        }
    }
    if (is_timedout(i2c)) {
        return -1;
    }

    writereg(IC_ENABLE(port), 0);

    writereg(IC_CON(port),
            IC_CON_SLAVE_DISABLE | IC_CON_RESTART_EN |
            IC_CON_SPEED_STANDARD | IC_CON_MASTER_MODE);

    writereg(IC_INTR_MASK(port), 0);

    writereg(IC_SS_HCNT(port), SS_SCL_HCNT);
    writereg(IC_SS_LCNT(port), SS_SCL_LCNT);

    i2c->target = target;
    writereg(IC_TAR(port), target);

    return 0;
}

static int
palo_i2c_do_master_read_write(struct palo_i2c *i2c, int is_read,
        u_int8_t *adbuf, int adlen,
        u_int8_t *buf, int len)
{
    int adcnt, txcnt, rxcnt, rxlen, nstarts, maxstarts, port;
    u_int32_t intr, txfsz;

    port = i2c->port;

    writereg(IC_ENABLE(port), 1);

    adcnt = txcnt = rxcnt = 0;
    rxlen = is_read ? len : 0;
    maxstarts = is_read ? 2 : 1;
    nstarts = 0;

    txfsz = ((readreg(IC_COMP_PARAM_1(port)) >> 16) & 0xff) + 1;
    writereg(IC_TX_TL(port), txfsz - 1);
    writereg(IC_RX_TL(port), 0);

    while (!is_timedout(i2c)) {
        intr = readreg(IC_RAW_INTR_STAT(port));
        if (intr & IC_INTR_RX_FULL) {
            while (rxcnt < rxlen && (readreg(IC_STATUS(port)) & IC_STATUS_RFNE))
                buf[rxcnt++] = readreg(IC_DATA_CMD(port));
        }
        if (intr & IC_INTR_RX_OVER)
            return (PALO_I2C_ERR_RX_OVER);
        if (intr & IC_INTR_TX_ABRT)
            return (PALO_I2C_ERR_TX_ABORT);
        if (intr & IC_INTR_START_DET) {
            if (++nstarts > maxstarts)
                return (PALO_I2C_ERR_TX_SPLIT_START);
            readreg(IC_CLR_START_DET(port));
        }
        if (intr & IC_INTR_STOP_DET) {
            if (txcnt != len || rxcnt != rxlen)
                return (PALO_I2C_ERR_TX_SPLIT_STOP);
        }
        if (intr & IC_INTR_TX_EMPTY) {
            if (adcnt < adlen) {
                writereg(IC_DATA_CMD(port), IC_DATA_CMD_WRITE | adbuf[adcnt]);
                ++adcnt;
            } else if (txcnt < len && readreg(IC_TXFLR(port)) < txfsz) {
                if (is_read)
                    writereg(IC_DATA_CMD(port), IC_DATA_CMD_READ);
                else
                    writereg(IC_DATA_CMD(port), IC_DATA_CMD_WRITE | buf[txcnt]);
                ++txcnt;
            }
        }
        if (txcnt == len && rxcnt == rxlen) {
            if (intr & IC_INTR_ACTIVITY)
                return (0);
        }
    }
    return (PALO_I2C_ERR_TIMEOUT);
}

static int
palo_i2c_master_read_write(struct palo_i2c *i2c, struct i2c_msg *msg, int num)
{
    int adlen, len, port, is_read, ret;
    u_int8_t *adbuf, *buf;

    reset_timer();
    i2c->xfer_timeout = sa_timer_get() + i2c->timeout;

    if (msg[0].addr != i2c->target || i2c->error) {
        if (palo_i2c_master_initialize(i2c, msg[0].addr) < 0) {
            i2c->error = 1;
            return (PALO_I2C_ERR_TIMEOUT);
        }
    }
    i2c->error = 0;

    if (num == 1) {
        adbuf = NULL;
        adlen = 0;
        buf = msg[0].buf;
        len = msg[0].len;
        is_read = msg[0].flags & I2C_M_RD;
    } else {
        adbuf = msg[0].buf;
        adlen = msg[0].len;
        buf = msg[1].buf;
        len = msg[1].len;
        is_read = msg[1].flags & I2C_M_RD;
    }

    ret = palo_i2c_do_master_read_write(i2c, is_read, adbuf, adlen, buf, len);

    port = i2c->port;
    while (!is_timedout(i2c)) {
        if (!(readreg(IC_STATUS(port)) & IC_STATUS_MST_ACTIVITY)) {
            writereg(IC_ENABLE(port), 0);
            return (ret);
        }
    }
    i2c->error = 1;
    return (PALO_I2C_ERR_TIMEOUT);
}

static int
palo_i2c_doxfer(struct palo_i2c *i2c, struct i2c_msg *msg, int num)
{
    const char *err = NULL;
    int ret;

    ret = palo_i2c_master_read_write(i2c, msg, num);
    switch (ret) {
    case PALO_I2C_ERR_RX_OVER:
        err = "rx-overrun";
        ret = -EAGAIN;
        break;

    case PALO_I2C_ERR_TX_ABORT:
        (void)readreg(IC_CLR_TX_ABRT(i2c->port));
        ret = -ENXIO;
        break;

    case PALO_I2C_ERR_TX_SPLIT_START:
        err = "split-transaction-start";
        ret = -EAGAIN;
        break;

    case PALO_I2C_ERR_TX_SPLIT_STOP:
        err = "split-transaction-stop";
        ret = -EAGAIN;
        break;

    case PALO_I2C_ERR_TIMEOUT:
        err = "timeout";
        ret = -EIO;
        break;

    default:
        break;
    }

    if (err != NULL)
        printf("target 0x%x: %s error\n", i2c->target, err);

    return ret;
}

static void
palo_i2c_init(void)
{
    struct palo_i2c *i2c = &gi2c;
    static int init_done = 0;

    if (init_done)
        return;
    i2c->port = 0;
    i2c->target = -1;
    i2c->retries = 3;
    i2c->timeout = PALO_I2C_TIMEOUT_MS * 1000000ULL;

    init_done = 1;
}

int
palo_i2c_xfer(struct i2c_msg *msg, int num)
{
    struct palo_i2c *i2c = &gi2c;
    u_int32_t flags;
    int i, ret = -EINVAL;

    palo_i2c_init();

    if (num != 1 && num != 2)
        return -EINVAL;
    if (msg[0].len == 0)
        return -EINVAL;
    flags = msg[0].flags;
    if (num == 2) {
        /* 
         * Two part transactions must be to the same slave,
         * must begin with a 1 or 2 byte write.
         */
        if (msg[1].addr != msg[0].addr || (msg[0].flags & I2C_M_RD) ||
            msg[1].len == 0 || (msg[0].len != 1 && msg[0].len != 2)) {
            return -EINVAL;
        }
        flags |= msg[1].flags;
    }

    /*
     * Only the I2C_M_RD flag is supported.
     */
    if (flags & ~I2C_M_RD)
        return -EINVAL;

    for (i = 0; i < i2c->retries; i++) {
        ret = palo_i2c_doxfer(i2c, msg, num);
        if (ret >= 0 || ret == -EIO)
            return ret;
    }

    return ret;
}
