
/*
 * Copyright (c) 2008 Cisco Systems, Inc.  All rights reserved.
 */
#ident "$Id: mezzi2c.h 46 2016-12-03 23:17:24Z buildsa $"

#ifndef _MEZZI2C_H_
#define _MEZZI2C_H_

#ifndef PACKED
#define PACKED  __attribute__((packed))
#endif

/*
 * I2C Devices on Mezz Cards - Addresses and Data Structures.
 */


/*
 * DEVICE ADDRESSES
 * Most devices include a 'c' component which corresponds to the
 * mezz card address (0 or 1).
 * On Monterey Park this bit is always 1.
 */

/*
 * I2C Bus Switch (arbiter).
 */
#define I2C_ARBITER_TARGET_ADDR(c)      (0x70 | (c))

/*
 * FRU ID PROM.
 */
#define I2C_FRU_TARGET_ADDR(c)          (0x52 | (c))

/*
 * POST PROM - used for POST and other BMC to MEZZ I/O.
 */
#define I2C_POST_TARGET_ADDR(c)         (0x56 | (c))

/*
 * Alternate Message PROM.
 * The alternate PROM is for older mezz cards.  This is their FRUID PROM.
 */
#define I2C_ALTPOST_TARGET_ADDR(c)      (I2C_FRU_TARGET_ADDR(c))

/*
 * GPIO Device.
 */
#define I2C_GPIO_TARGET_ADDR(c)         (0x24 | (c))

/*
 * Thermal Monitor.
 * (fixed address)
 */
#define I2C_THERM_TARGET_ADDR(c)        (0x1c)

/*
 * PEX8632 PCI Express Switch (Monterey Park).
 * (fixed address)
 */
#define I2C_PEX_TARGET_ADDR(c)          (0x68)

/*
 * Slave Device Address in SFP
 */
#define I2C_SFP_SLAVE_ADDR0              (0x50)
#define I2C_SFP_SLAVE_ADDR1              (0x51)

/*
 * FRUPROM DATA STRUCTURES
 * Defined in src/common/libfru/include/ipmi_sprom.h
 */

/*
 * POSTPROM DATA STRUCTURES
 */

/*
 * POST Structure.
 * The post code itself is a 32-bit big-endian integer bracketed by two
 * instances of the record sequence number.  The sequence number changes
 * every time the post is written.  Both seq1 and seq2 must be identical for
 * the record to be considered valid.
 */
#define I2C_POST_BASE_ADDRESS           0x400
#ifndef __ASSEMBLY__
typedef struct {
    u_int8_t    seq1;                   // sequence number
    u_int32_t   post;                   // big-endian post code
    u_int8_t    seq2;                   // copy of seq1
} PACKED mezzpost_t;
#endif



/*
 * Boot Control Structure.
 * The boot control structure
 */
#define I2C_BOOTCTRL_BASE_ADDRESS       0x500
#ifndef __ASSEMBLY__
typedef struct {
    u_int32_t   cmd;                    // command flags
    u_int32_t   magic;                  // magic number
} PACKED mezzbootctrl_t;
#endif
#define BOOTCMD_MAGIC       0x42434d44  // 'B', 'C', 'M', 'D'

/*
 * Command flags.
 */
#define BOOTCMD_LOAD_ALTAPP     0x1     // load alternate firmware
#define BOOTCMD_LOAD_DIAG       0x2     // load manufacturing diags
#define BOOTCMD_OPT_DIAGMODE    0x10    // run firmware in diag mode
#define BOOTCMD_OPT_NOVNICS     0x20    // do not apply vnic config
#define BOOTCMD_OPT_NOCONFIG    0x40    // do not apply other config
#define BOOTCMD_OPT_NOPOST      0x80    // do not perform POST
#define BOOTCMD_OPT_DEBUGVNIC   0x100   // install a debug vnic
#define BOOTCMD_OPT_MEMTUN      0x200   // enable memtun access
#define BOOTCMD_OPT_JFFS2RESET  0x400   // clear /config and /obfl
#define BOOTCMD_OPT_AMPENABLE   0x800   // enable AMP protocol
#define BOOTCMD_OPT_DBGDUMP     0x1000  // force VIC debug dump

/*
 * Card (Monterey Park) Config Block.
 * This structure is for PCIe cards only, but it's called mezzxxx for
 * consistency with other types defined here.
 */
#define I2C_CONFIGBLOCK_BASE_ADDRESS    0x580
#ifndef __ASSEMBLY__
typedef struct {
    u_int32_t magic;                    // magic number
    u_int8_t  version;                  // format version
    u_int8_t  length;                   // struct length inclusive
    u_int8_t  checksum;                 // twos complement checksum (sum to 0)
    u_int8_t  ncsi_pkgid;               // ncsi package id
    u_int8_t  ncsi_fast;                // ncsi fast mode enable
    u_int8_t  ncsi_vlan_mode;           // ncsi vlan mode
    u_int16_t ncsi_vlan;                // ncsi vlan id
    u_int8_t  ncsi_vlan_privacy;        // ncsi vlan privacy enable
    u_int8_t  pci_slot_id;              // installed server pci slot id
    u_int8_t  ucsm_mode;                // UCSM mode flag
    u_int8_t  pcielinkwidth;            // Width of the PCIe link
    u_int8_t  pcie_linktype;            // PCIe link type (straight or swizzled)
} PACKED mezzconfigblock_t;
#endif

#define MEZZCONFIG_HDRLEN   7           // magic+version+length+checksum
#define MEZZCONFIG_MAGIC    0x4d504347
#define MEZZCONFIG_VERSION  0x1
#define MEZZCONFIG_MIN_LENGTH 15
#define MEZZCONFIG_MAX_LENGTH 255

#define MEZZCONFIG_UCSM_MODE_UCSM       0x1  // bit-0
#define MEZZCONFIG_UCSM_MODE_UCSM_NIV   0x2  // bit-1 valid if bit-0 is set

/*
 * The pci_slot_id field is encoded.  If bit 7 is clear, the code is an
 * ASCII character.  If bit 7 is set then bits 6:0 represent an integer.
 * This allows slots to be identified either by letter or (big) number.
 */
#define MEZZCONFIG_PCI_SLOT_NUMF        0x80
#define MEZZCONFIG_PCI_SLOT_IS_NUM(x)   ((x) & 0x80)
#define MEZZCONFIG_PCI_SLOT_NUM(x)      ((x) & 0x7f)

/*
 * The pcie slot width represents Number of PCIe lane valid in current slot.
 * Supported values for pcielinkwidth
 * are x4 : 1
 *     x8 : 2
 *     x16: 3
 */
#define MEZZCONFIG_PCIWIDTH_X4    1
#define MEZZCONFIG_PCIWIDTH_X8    2
#define MEZZCONFIG_PCIWIDTH_X16   3
#define MEZZCONFIG_PCIWIDTH_DEFAULT MEZZCONFIG_PCIWIDTH_X8

/*
 * VIC PCIe lane0 in some server (for e.g DN2 and DN1 x16 slots) is wired
 * physically to lane15 of server CPU.
 * Supported values for pcie_linktype are:
 * Straight : VIC(PCIElane[0..15]) <-> SERVER_CPU(PCIElane[0..15]) : 1
 * Swizzled : VIC(PCIElane[0..15]) <-> SERVER_CPU(PCIElane[15..0]) : 2
 */
#define MEZZCONFIG_PCILINKTYPE_STRAIGHT 1
#define MEZZCONFIG_PCILINKTYPE_SWIZZLED 2
#define MEZZCONFIG_PCILINKTYPE_DEFAULT MEZZCONFIG_PCILINKTYPE_STRAIGHT

/*
 * CIMC Scratch Area.
 * This area is reserved for the CIMC.
 */
#define MEZZCIMC_SCRATCH_BASE_ADDRESS   0x800
#define MEZZCIMC_SCRATCH_LEN            0x400

/*
 * EEPROM page write size.
 */
#define I2C_EEPROM_PAGESIZE             32

#endif
