
/*
 * Copyright (c) 2007 Nuova Systems, Inc.  All rights reserved.
 */
#ident "$Id: ecpuctx.h 43 2016-12-03 22:59:19Z buildsa $"

#ifndef _ECPUCTX_H_
#define _ECPUCTX_H_

/*
 * This file describes the eCPU context in low physical memory on PALO.
 *
 * The first 8K in physical memory is reserved for eCPU boot vectors and
 * arguments (1968 bytes per ecpu).  This supports 5 eCPUs (incl. mcpu) which
 * is sufficient for both Cruz (5) and Palo/Sereno (3).
 *
 * 0x2000 +---------------+ ---
 *        |               |  ^
 *        |   eCPU4 arg   |  |
 *        |               |  |
 * 0x1850 +---------------+  | Cruz only
 *        |               |  |
 *        |   eCPU3 arg   |  |
 *        |               |  v
 * 0x10a0 +---------------+ ---
 *        |               |
 *        |   eCPU2 arg   |
 *        |               |
 * 0x08f0 +---------------+
 *        |               |
 *        |   eCPU1 arg   |
 *        |               |
 * 0x0140 +---------------+ ---
 *        | eCPU4 context |  ^
 * 0x0100 +---------------+  | Cruz only
 *        | eCPU3 context |  v
 * 0x00c0 +---------------+ ---
 *        | eCPU2 context |
 * 0x0080 +---------------+
 *        | eCPU1 context |
 * 0x0040 +---------------+
 *        | eCPU0 context |
 * 0x0000 +---------------+
 *
 * The eCPU context area provides the following functions:
 *      + Reset (vector + argument pointer).
 *      + NMI (vector).
 *      + Debug (vector + saved k0, k1).
 *
 * The eCPU arg area provides holds the boot argument for the dataplane eCPUs.
 * The eCPUs should not assume the address, but rather should use the supplied
 * a0 register provided at startup.
 */
#define ECPUCTX_MEMBASE     0x0         // Required to be zero
#define ECPUCTX_MEMSIZE     8192

#define ECPUCTX_MAGIC       0x55504365  // 'eCPU'

#define ECPUCTX_LOG2SIZE    6           // sizeof(ecpuctx_t) = (1 << 6)

#define ECPUCTX_NMIMAGIC    0x494d4e65  // 'eNMI'
#define ECPUCTX_DBGMAGIC    0x47424465  // 'eDBG'

#ifndef __ASSEMBLY__

/*
 * When taking an NMI exception:
 *   + MIPS registers are set per NMI Exception spec.
 *   + k0, k1 and t1 are clobbered, but saved in nmik0, nmik1 and nmitmp.
 *   + k0 points to this per-eCPU struct in kseg1.
 *
 * When taking a Debug exception:
 *   + MIPS registers are set per Debug Exception spec.
 *   + k0 and k1 are clobbered, but saved in dbgk0 and dbgk1.
 *   + k0 points to this per-eCPU struct in kseg1.
 */
typedef struct {
    u_int32_t       magic;              // magic number
    u_int32_t       entry;              // entry address
    u_int32_t       arg;                // a0 register
    u_int32_t       nmimagic;           // nmi vector magic
    u_int32_t       nmivec;             // nmi vector
    u_int32_t       nmitmp;             // scratch
    u_int32_t       dbgmagic;           // debug vector magic
    u_int32_t       dbgvec;             // debug vector
    u_int32_t       dbgk0;              // debug saved k0
    u_int32_t       dbgk1;              // debug saved k1
    u_int32_t       nmik0;              // nmi saved k0
    u_int32_t       nmik1;              // nmi saved k1
    u_int32_t       vcons;              // virtual console phys addr
    u_int32_t       dbpage;             // debug page KSEG1 addr
    u_int32_t       pad[2];             // pad to 64 bytes
} ecpuctx_t;

/*
 * Memory map.
 * 5 x contexts + 4 x argument areas.
 */
#define ECPUCTX_NCPUS   5               // max of Cruz and Palo/Sereno
#define ECPUCTX_NARGS   (ECPUCTX_NCPUS - 1) // no arg area for mcpu

#define ECPUCTX_ARGSIZE \
        ((ECPUCTX_MEMSIZE - ECPUCTX_NCPUS * sizeof(ecpuctx_t)) / ECPUCTX_NARGS)

typedef struct {
    ecpuctx_t       ctx[ECPUCTX_NCPUS];
    u_int8_t        arg[ECPUCTX_NARGS][ECPUCTX_ARGSIZE];
} ecpupage_t;

#else   /* __ASSEMBLY__ */

/*
 * ecpu_ctx_t offsets.
 */
#define ECTX_MAGIC          0x00
#define ECTX_ENTRY          0x04
#define ECTX_ARG            0x08
#define ECTX_NMIMAGIC       0x0c
#define ECTX_NMIVEC         0x10
#define ECTX_NMITMP         0x14
#define ECTX_DBGMAGIC       0x18
#define ECTX_DBGVEC         0x1c
#define ECTX_DBGK0          0x20
#define ECTX_DBGK1          0x24
#define ECTX_NMIK0          0x28
#define ECTX_NMIK1          0x2c
#define ECTX_VCONS          0x30
#define ECTX_DBPAGE         0x34

#endif      /* __ASSEMBLY__ */

#endif

