
/*
 * Copyright 2012 Cisco Systems, Inc.  All rights reserved.
 */
#ident "$Id: arb.c 21 2016-12-02 21:25:59Z buildsa $"

#ifdef __UBOOT__
#include <common.h>
#include <command.h>
#include <asm/addrspace.h>
#include <asm/io.h>
#include <palo/fw/mcpu/include/pca9541.h>
#include <palo/fw/mcpu/include/board_conf.h>
#include "palodtls.h"

#define LOG_DECL(x)
#define log_info(x, ...) do { } while (0)
#define log_error(x, ...) do { } while (0)
#define i2c_logf(x, ...) do { } while (0)

#else

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include "log.h"
#include "libi2c.h"
#include "libarb.h"
#include "pca9541.h"
#include "board_conf.h"
#include "sa_timer.h"
#endif

LOG_DECL(i2carb);

#define ARB_MAX_ACQUIRE_ATTEMPTS    40          // 40 x 250ms = 10s max
#define ARB_MAX_TRANSFER_ATTEMPTS   3

#define ARB_MASTER_RETRY_TIME_US    250000
#define ARB_NTESTON_WAIT_TIME_US    50000
#define ARB_BUSINIT_DELAY_US        250

static int
arb_read(int port, u_int8_t reg, u_int8_t *valp, u_int8_t cnt)
{
    if (cnt > 1)
        reg |= ARB_AUTOINC;
    return (i2c_master_access(port, I2C_OP_READ,
            reg, I2C_ADDR_WIDTH_8, valp, cnt, 0));
}

static int
arb_write(int port, u_int8_t reg, u_int8_t *valp, u_int8_t cnt)
{
    if (cnt > 1)
        reg |= ARB_AUTOINC;
    return (i2c_master_access(port, I2C_OP_WRITE,
            reg, I2C_ADDR_WIDTH_8, valp, cnt, 0));
}

static int
is_buson(u_int8_t con)
{
    /* Return 1 if ((BUSON ^ NBUSON) != 0) */
    return (!(con & ARB_CON_BUSON) ^ !(con & ARB_CON_NBUSON));
}

static int
is_mybus(u_int8_t con)
{
    /* Return 1 if ((MYBUS ^ NMYBUS) == 0) */
    return (!(!(con & ARB_CON_MYBUS) ^ !(con & ARB_CON_NMYBUS)));
}

static int
is_acquired(u_int8_t con)
{
    return (is_buson(con) && is_mybus(con));
}

static int
is_peer_claimed(u_int8_t istat)
{
    return (istat & ARB_ISTAT_NMYTEST);
}

static int
is_peer_acquired(u_int8_t con)
{
    return (is_buson(con) && !is_mybus(con));
}

static int
is_peer_claimed_or_acquired(u_int8_t con, u_int8_t istat)
{
    return (is_peer_claimed(istat) || is_peer_acquired(con));
}

#ifndef __UBOOT__
static void
udelay(u_int32_t us)
{
    u_int64_t t = sa_timer_get() + us * 1000;

    while ((int)(t - sa_timer_get()) > 0) {
        /* spin */ ;
    }
}
#endif

#define e_arb_read(a, b, c, d)              \
    do {                                    \
        int ret = arb_read(a, b, c, d);     \
        if (ret < 0) {                      \
            return ret;                     \
        }                                   \
    } while (0)

#define e_arb_write(a, b, c, d)             \
    do {                                    \
        int ret = arb_write(a, b, c, d);    \
        if (ret < 0) {                      \
            return ret;                     \
        }                                   \
    } while (0)

static int
arb_connect_workaround(int port, u_int8_t ncon)
{
    u_int8_t con, istat;
    int try;

    for (try = 0; try < 2; try++) {
        /*
         * Write CON.  Wait 250us for BUSINIT to complete and then read ISTAT.
         * If ISTAT has ARB_ISTAT_BUSINIT set then success.
         */
        e_arb_write(port, ARB_CON, &ncon, 1);
        udelay(ARB_BUSINIT_DELAY_US);
        e_arb_read(port, ARB_ISTAT, &istat, 1);

        /*
         * We're done if BUSINIT occurred or if this is the 2nd attempt.
         * If BUSINIT didn't happen we return success here.  Let the higher
         * level retry.
         */
        if (try == 1 || (istat & ARB_ISTAT_BUSINIT)) {
            break;
        }

        /*
         * BUSINIT failed.  This is the 9541 internal oscillator startup
         * failure signature.  The workaround is to toggle the CON register
         * a couple of times.
         * First write:  ~BUSON, ~BUSINIT
         * Second write: BUSON, BUSINIT
         */
        con = ncon & ~ARB_CON_BUSINIT;
        con ^= ARB_CON_BUSON;
        e_arb_write(port, ARB_CON, &con, 1);
    }
    return (0);
}

static int
arb_write_con_standard(int port, u_int8_t ncon)
{
    e_arb_write(port, ARB_CON, &ncon, 1);

    /*
     * Wait 250us before proceeding after a BUSINIT
     */
    if (ncon & ARB_CON_BUSINIT) {
        udelay(ARB_BUSINIT_DELAY_US);
    }

    return (0);
}

static int
arb_write_con(int port, u_int8_t ncon)
{
    if ((ncon & ARB_CON_BUSINIT) && board_conf_do_9541_workaround()) {
        /*
         * This is an attempt to connect the bus on a system that might
         * be susceptible to the 9541 errata.  Use workaround-savvy logic.
         */
        return (arb_connect_workaround(port, ncon));
    } else {
        /*
         * Non-errata part.
         * Normal write/read.
         */
        return (arb_write_con_standard(port, ncon));
    }
}

#define e_arb_write_con(a, b)               \
    do {                                    \
        int ret = arb_write_con(a, b);      \
        if (ret < 0) {                      \
            return ret;                     \
        }                                   \
    } while (0)

static int
arb_acquire_bus(int port, u_int8_t arb_target_addr)
{
    u_int8_t con, ocon, istat;
    u_int8_t reg[2];
    int i, ret;

    /*
     * Direct the I2C controller to access the arbiter.
     */
    i2c_master_initialize(port, arb_target_addr, ARB_MAX_TRANSFER_ATTEMPTS);

    for (i = 0; i < ARB_MAX_ACQUIRE_ATTEMPTS; i++) {
        /*
         * First read the arbiter state to see who has the bus.
         */
        e_arb_read(port, ARB_CON, reg, 2);
        con = reg[0];
        istat = reg[1];

        /*
         * Check both NMYTEST and also the bus ownership state
         * to determine if the bus has been requested or acquired
         * by the other peer.
         */
        if (!is_peer_claimed_or_acquired(con, istat)) {
            /*
             * Bus free.  State intent.
             */
            con = (con & ~ARB_CON_BUSINIT) | ARB_CON_NTESTON;
            e_arb_write_con(port, con);

            /*
             * Sleep 50ms to allow peer to observe NTESTON.
             */
            usleep(ARB_NTESTON_WAIT_TIME_US);

            /*
             * Read CON again to see if an old peer acquired the bus.
             */
            e_arb_read(port, ARB_CON, reg, 1);

            /*
             * If there's no collision (CON state changed)
             * and the peer has not taken the bus then it is ours.
             */
            if (reg[0] == con && !is_peer_acquired(con)) { 
                /*
                 * We have the bus.  Determine how to set BUSON and MYBUS.
                 * If there is nothing to do then return.
                 */
                ocon = con;
                if (!is_buson(con))
                    con ^= ARB_CON_BUSON;
                if (!is_mybus(con))
                    con ^= ARB_CON_MYBUS;
                if (con == ocon)
                    return (0);

                /*
                 * Write the control register to take the bus; performing a
                 * BUSINIT on the way, and verifying that the value we write
                 * sticks.
                 */
                con |= ARB_CON_BUSINIT;
                ret = arb_write_con(port, con);
                if (ret < 0) {
                    return (ret);
                }

                /*
                 * The value we wrote stuck, so we now own the bus.
                 * Clear BUSINIT.
                 */
                con &= ~ARB_CON_BUSINIT;
                e_arb_write_con(port, con);
                return (0);
            }

            /*
             * Conflict.  Clear NTESTON if it is safe.
             * If an old peer acquired the bus then leave CON alone.
             */
            if (reg[0] == con) {
                con = reg[0] & ~ARB_CON_NTESTON;
                e_arb_write_con(port, con);
            }
        }

        /*
         * Sleep to let the other master do their transfer and then
         * read the control register for the next loop.
         */
        usleep(ARB_MASTER_RETRY_TIME_US);
    }

    return (I2C_ERR_MAXTRIES);
}

static int
arb_release_bus(int port, u_int8_t arb_target_addr)
{
    u_int8_t con, ncon;
    int ret = 0;

    i2c_master_initialize(port, arb_target_addr, ARB_MAX_TRANSFER_ATTEMPTS);

    /*
     * First read the control register to see if we have the bus.
     * If we do, then flip BUSON to release it.
     * Also ensure BUSINIT and clear NTESTON.
     */
    e_arb_read(port, ARB_CON, &con, 1);
    ncon = con & ~(ARB_CON_BUSINIT | ARB_CON_NTESTON);
    if (is_acquired(con)) {
        ncon ^= ARB_CON_BUSON;
    }
    if (ncon != con) {
        ret = arb_write_con(port, ncon);
    }
    
    return (ret);
}



/*
 * Determine if the bus has been lost.
 * Return 1 or 0, or -ve error code.
 */
static int
arb_is_buslost(int port, u_int8_t arb_target_addr)
{
    u_int8_t reg[2];
    u_int8_t con, istat;

    i2c_master_initialize(port, arb_target_addr, ARB_MAX_TRANSFER_ATTEMPTS);
    e_arb_read(port, ARB_CON, reg, 2);

    con = reg[0];
    istat = reg[1];

    /*
     * If the bus is no longer ours and connected, or the interrupt register
     * reports BUSLOST, then return true.
     */
    if (!is_acquired(con) || (istat & ARB_ISTAT_BUSLOST)) {
        return 1;
    } else {
        return 0;
    }
}



/*
 * Perform a protected transfer on the shared bus.
 *
 * Acquire the arbiter using the published algorithm.  When the bus is
 * acquired, call the user transfer function to do their work.  Finally
 * release the bus.
 */
int
arb_protected_transfer(int port, u_int8_t arb_target_addr, int nattempts,
        int (*xfer_func)(void *opaque), void *opaque)
{
    int ret, uret, i;
    
    uret = 0;
    for (i = 0; i < nattempts; i++) {
        i2c_logf(port, "arb_protected_transfer i=%d", i);
        ret = arb_acquire_bus(port, arb_target_addr);
        if (ret < 0) {
            log_error("Failed to acquire I2C bus (err %d)", ret);
            return (ret);
        }

        /*
         * Call the user function to perform their transfer.  This may
         * fail if the bus is stolen, but I've also observed failures
         * such as tx-abort with the bus still apparently connected.
         * So retry on any failure.
         */
        uret = (*xfer_func)(opaque);
        if (uret < 0) {
            i2c_logf(port, "arb_protected_transfer uret=%d", uret);
        }

        ret = arb_is_buslost(port, arb_target_addr);
        if (ret >= 0) {
            if (ret) {
                log_info("I2C bus lost");
            } else {
                arb_release_bus(port, arb_target_addr);
            }
            if (ret == 0 && uret == 0) {
                return (uret);
            }
        }
        usleep(ARB_MASTER_RETRY_TIME_US);
    }
    log_error("Protected bus transfer retries exceeded");
    return (uret < 0) ? uret : I2C_ERR_MAXTRIES;
}
