
/*
 * Copyright (c) 2008 Cisco Systems, Inc.  All rights reserved.
 */
#ident "$Id: i2c.c 43 2016-12-03 22:59:19Z buildsa $"

#ifdef __UBOOT__
#include <common.h>
#include <command.h>
#include <asm/addrspace.h>
#include <asm/io.h>
#include <palo/include/misc_mregs.h>
#include <palo/fw/mcpu/include/board_conf.h>
#include "palodtls.h"

#define LOG_DECL(x)
#define log_info(x, ...) do { } while (0)
#define log_error(x, ...) do { } while (0)
#define i2c_logf(x, ...) do { } while (0)

#else

#include <sys/types.h>
#include <sys/ioctl.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <stdarg.h>
#include <errno.h>
#include <fcntl.h>
#include <linux/i2c-dev.h>
#include "misc_mregs.h"
#include "board_conf.h"
#include "sa_timer.h"
#include "regio.h"
#include "log.h"
#include "liblock.h"
#include "libi2c.h"

/*
 * XXX: copied from /usr/include/linux/i2c.h.
 */
struct i2c_msg {
        __u16 addr;     /* slave address                        */
        __u16 flags;
        __u16 len;              /* msg length                           */
        __u8 *buf;              /* pointer to msg data                  */
};
#define I2C_M_RD        0x01
#define I2C_RDWR        0x0707  /* Combined R/W transfer (one stop only)*/

#endif

LOG_DECL(i2c);

static struct {
    u_int16_t   target;
    int         nattempts;
} g_port[I2C_NPORTS];

/*
 * Define I2C_LOCKING to support concurrent app use of the I2C bus.
 */
#ifdef I2C_LOCKING
static lockobj_t *i2c_lp = NULL;
static int i2c_initdone = 0;
#endif

#if defined __UBOOT__
int errno;

static int
i2c_dev_open(int port)
{
    return port;
}

static int
i2c_dev_op(int fd, struct i2c_rdwr_ioctl_data *dat)
{
    int ret;

    ret = palo_i2c_xfer(dat->msgs, dat->nmsgs);
    if (ret < 0) {
        errno = -ret;
        ret = -1;
    }
    return ret;
}

static void
i2c_dev_close(int fd)
{
}
#else
static int
i2c_dev_open(int port)
{
    char path[20];
    int fd;

    snprintf(path, sizeof (path), "/dev/i2c-%d", port);
    fd =  open(path, O_RDWR, 0);
    if (fd < 0) {
        log_error("%s: %s", path, strerror(errno));
    }
    return fd;
}

static int
i2c_dev_op(int fd, struct i2c_rdwr_ioctl_data *dat)
{
    return ioctl(fd, I2C_RDWR, dat);
}

static void
i2c_dev_close(int fd)
{
    close(fd);
}
#endif

/*
 * Spinning ns delay.
 */
static void
spinwait_ns(u_int64_t ns)
{
    u_int64_t t_start, t_now;

    t_start = sa_timer_get();
    do {
        t_now = sa_timer_get();
    } while (t_now - t_start < ns);
}

static void
i2c_controller_reset(void)
{
    u_int32_t val;

    /*
     * Reset the I2C controller by toggling the I2C bit in the
     * misc_sw_hard_rst register.
     */
    log_info("resetting i2c controller");
    val = readreg(MISC_MREGS_SW_HARD_RST);
    val |= MISC_MREGS_SW_HARD_RST_I2C;
    writereg(MISC_MREGS_SW_HARD_RST, val);
    spinwait_ns(5000);
    val &= ~MISC_MREGS_SW_HARD_RST_I2C;
    writereg(MISC_MREGS_SW_HARD_RST, val);
    spinwait_ns(5000);
}

#ifdef CONFIG_BASE_PALO
/*
 * Palo overrides the clock on port 0 via a GPIO.
 */
static int
is_i2c_override_capable(int port)
{
    return (port == 0 && board_conf_has_i2c0reset());
}

static void
i2c_override_begin(int port)
{
    if (port == 0) {
        board_conf_begin_i2c0reset();
    }
}

static void
i2c_override_set(int port, int scl, int sda)
{
    if (port == 0) {
        board_conf_set_i2c0reset(scl);
    }
}

static void
i2c_override_end(int port)
{
    if (port == 0) {
        board_conf_end_i2c0reset();
    }
}
#elif defined(CONFIG_BASE_SERENO)
/*
 * Sereno uses built-in pad overrides on both i2c ports.
 */
static int
is_i2c_override_capable(int port)
{
    return 1;
}

static void
i2c_override_begin(int port)
{
    /* no op */
}

static void
i2c_override_set(int port, int scl, int sda)
{
    u_int32_t addr = MISC_MREGS_BE_I2C_CONTROL0 +
            (MISC_MREGS_BE_I2C_CONTROL1 - MISC_MREGS_BE_I2C_CONTROL0) * port;
    u_int32_t val;

    misc_mregs_be_i2c_control0_make(&val,
          1     /* pad_override */,
          sda   /* pad_override_data */,
          scl   /* pad_override_clk */,
          1     /* pad_clk_override */);
    writereg(addr, val);
}

static void
i2c_override_end(int port)
{
    u_int32_t addr = MISC_MREGS_BE_I2C_CONTROL0 +
            (MISC_MREGS_BE_I2C_CONTROL1 - MISC_MREGS_BE_I2C_CONTROL0) * port;
    writereg(addr, 0);
}
#elif defined (CONFIG_BASE_CRUZ)
/*
 * Cruz uses built-in pad overrides on both i2c ports.
 */
static int
is_i2c_override_capable(int port)
{
    return 1;
}

static void
i2c_override_begin(int port)
{
    /* no op */
}

static void
i2c_override_set(int port, int scl, int sda)
{
    u_int32_t addr = MISC_MREGS_I2C_CONTROL0 +
            (MISC_MREGS_I2C_CONTROL1 - MISC_MREGS_I2C_CONTROL0) * port;
    u_int32_t val;

    misc_mregs_i2c_control0_make(&val,
          1     /* pad_override */,
          sda   /* pad_override_data */,
          scl   /* pad_override_clk */,
          1     /* pad_clk_override */);
    writereg(addr, val);}

static void
i2c_override_end(int port)
{
    u_int32_t addr = MISC_MREGS_I2C_CONTROL0 +
            (MISC_MREGS_I2C_CONTROL1 - MISC_MREGS_I2C_CONTROL0) * port;
    writereg(addr, 0);
}

#else
#error "CONFIG_BASE not defined"
#endif

/*
 * Attempt to clear a stuck bus (PALO)
 * Toggle SCL up to 18 times (2 bytes @ 9 bits/byte).  Each time, reset the
 * I2C controller and then attempt to perform a dummy read operation to
 * target 0x7f.  If the transaction returns EIO then the bus did not clear.
 */
static int
i2c_clear_stuck_bus(int port, int fd)
{
    u_int8_t adbuf[1] = { 0 };
    u_int8_t buf[1];
    struct i2c_msg msg[2] = {
        {
            .addr = 0x7f,
            .flags = 0,
            .len = 1,
            .buf = adbuf
        },
        {
            .addr = 0x7f,
            .flags = I2C_M_RD,
            .len = 1,
            .buf = buf
        },
    };
    struct i2c_rdwr_ioctl_data dat = {
        .msgs = msg,
        .nmsgs = 2
    };
    int i, rc;

    if (!is_i2c_override_capable(port)) {
        return -1;
    }

    /*
     * Reset the entire controller and then toggle SCL.
     * Try that 18 times.
     */
    for (i = 0; i < 18; i++) {
        i2c_controller_reset();
        i2c_override_begin(port);
        i2c_override_set(port, 1, 1);
        spinwait_ns(5000);
        i2c_override_set(port, 0, 1);
        spinwait_ns(5000);
        i2c_override_end(port);
        rc = i2c_dev_op(fd, &dat);
        if (rc == 0 || errno != EIO) {
            return 0;
        }
    }
    return -1;
}

void
i2c_master_initialize(int port, u_int8_t target, int nattempts)
{
    if (port < 0 || port >= I2C_NPORTS)
        return;
    g_port[port].target = target;
    g_port[port].nattempts = nattempts;
}

int
i2c_master_access(int port, int op, u_int32_t addr,
        int addr_width, u_int8_t *buffer, int len, u_int32_t flags)
{
    static int width_to_bytes[] = {
        [I2C_ADDR_WIDTH_0] = 0,
        [I2C_ADDR_WIDTH_8] = 1,
        [I2C_ADDR_WIDTH_16] = 2,
        [I2C_ADDR_WIDTH_32] = 4,
    };
    u_int8_t adbuf[4];
    struct i2c_msg msg[2] = {
        {
            .addr = g_port[port].target,
            .flags = 0,
            .len = width_to_bytes[addr_width],
            .buf = adbuf
        },
        {
            .addr = g_port[port].target,
            .flags = (op == I2C_OP_READ) ? I2C_M_RD : 0,
            .len = len,
            .buf = buffer
        }
    };
    struct i2c_rdwr_ioctl_data dat = {
        .msgs = msg,
        .nmsgs = 2
    };
    int fd, rc, i, do_retry;

    if (port < 0 || port >= I2C_NPORTS || g_port[port].target == 0)
        return I2C_ERR_INVALID;

    fd = i2c_dev_open(port);
    if (fd < 0) {
        return I2C_ERR_TIMEOUT;
    }

    switch (addr_width) {
    case I2C_ADDR_WIDTH_8:
        adbuf[0] = addr;
        break;

    case I2C_ADDR_WIDTH_16:
        adbuf[0] = addr >> 8;
        adbuf[1] = addr;
        break;

    case I2C_ADDR_WIDTH_32:
        adbuf[0] = addr >> 24;
        adbuf[1] = addr >> 16;
        adbuf[2] = addr >> 8;
        adbuf[3] = addr;
        break;
    }

    i = 0;
    do {
        /*
         * If the operation fails with EIO (timeout) then it indicates that
         * the I2C controller failed to complete a transaction which might
         * indicate that it is hung.
         * If that happens reset the controller and try again.
         */
        do_retry = 0;
        rc = i2c_dev_op(fd, &dat);
        if (rc < 0) {
            switch (errno) {
            case EINVAL:
                log_error("i2c%d: %s", port, strerror(errno));
                rc = I2C_ERR_INVALID;
                break;

            case EIO:
                if (i2c_clear_stuck_bus(port, fd) == 0) {
                    log_info("i2c%d: stuck bus - cleared", port);
                    do_retry = 1;
                } else {
                    log_error("i2c%d: stuck bus - failed to clear", port);
                    rc = I2C_ERR_TIMEOUT;
                }
                break;

            case ENXIO:
                rc = I2C_ERR_TX_ABORT;
                do_retry = 1;
                break;

            default:
                rc = I2C_ERR_MAXTRIES;
                do_retry = 1;
                break;
            }
        }
    } while (do_retry && ++i < g_port[port].nattempts);
    i2c_dev_close(fd);

    return rc;
}

#if !defined __UBOOT__
void
i2c_logf(int port, const char *fmt, ...)
{
    char buf[32];
    va_list ap;
    FILE *fp;

    snprintf(buf, sizeof (buf), "/dev/i2clog%d", port);
    fp = fopen(buf, "w");
    if (fp == NULL) {
        return;
    }
    va_start(ap, fmt);
    vfprintf(fp, fmt, ap);
    va_end(ap);
    fclose(fp);
}
#endif

#ifdef I2C_LOCKING
static void
i2c_init(void)
{
    i2c_lp = lock_get(LOCK_CLASS_I2C);
    i2c_initdone = 1;
}

int
i2c_lock(void)
{
    if (!i2c_initdone)
        i2c_init();
    return (lock_acquire(i2c_lp));
}

void
i2c_unlock(void)
{
    if (!i2c_initdone)
        i2c_init();
    else
        lock_release(i2c_lp);
}
#else
int
i2c_lock(void)
{
    return (0);
}

void
i2c_unlock(void)
{
}
#endif
