/*
* Copyright 2012 Cisco Systems, Inc.  All rights reserved.
*/
#ident "$Id: palomem.h 46 2016-12-03 23:17:24Z buildsa $"

#ifndef _PALO_PALOMEM_H_
#define _PALO_PALOMEM_H_

/*
 * Layout of physical memory used by various components
 * of the ecpu firmware and associated applications.
 * Cruz has 512MB of memory, addressed as two non-contiguous 256MB blocks.
 * As the fwdtab, viftbl and genio regions are above 0x20000000 they cannot 
 * be accessed through KSEG0 or KSEG1.  Kernel access to these require an
 * explicit mapping (KSEG0/1 macros won't work).  User access to these via
 * /dev/mem are fine.
 *
 *             2GB Configuration
 *
 * 0x80000000 +-----------------+
 *            |                 |
 *            |      genio      | 1.44G
 *            |                 |
 * 0x26000000 +-----------------+
 *            |                 |
 *            |     usnicft     | 32M
 *            |                 |
 * 0x24000000 +-----------------+
 *            |                 |
 *            |     rdmaft      | 64M
 *            |                 |
 * 0x20000000 +-----------------+
 *            |                 |
 *            |     viftbl      | 96M
 *            |                 |
 * 0x1a000000 +-----------------+ 0x9a000000 alias
 *            |                 |
 *            |     fwdtab      | 24M
 *            |                 |
 * 0x18800000 +-----------------+ 0x98800000 alias
 *            |                 |
 *            |    linuxhi      | 136M
 *            |                 |
 * 0x10000000 +-----------------+ 0x90000000 alias
 *            |     pmpcie      | 16M
 * 0x0f000000 +-----------------+
 *            |      hwio       | 29M
 *            |                 |
 * 0x0d300000 +-----------------+
 *            |                 |
 *            |      ecpu4      | 32M
 *            |                 |
 * 0x0b300000 +-----------------+
 *            |                 |
 *            |      ecpu3      | 32M
 *            |                 |
 * 0x09300000 +-----------------+
 *            |                 |
 *            |      ecpu2      | 32M
 *            |                 |
 * 0x07300000 +-----------------+
 *            |                 |
 *            |      ecpu1      | 32M
 *            |                 |
 * 0x05300000 +-----------------+
 *            |      extbl      | 16M
 * 0x04300000 +-----------------+
 *            |                 |
 *            |      Linux      | 66M (+a bit)
 *            |                 |
 * 0x00043000 +-----------------+
 *            |     memtun      | 256K
 * 0x00003000 +-----------------+
 *            |      diag       | 4K
 * 0x00002000 +-----------------+
 *            |      ectx       | 8K
 * 0x00000000 +-----------------+
 */

/*
 * Define the top of the memory map, taking into account aliasing.
 * 2GB + The 0x10000000 alias at 0x90000000 - 0x9fffffff.
 */
#define PALOMEM_MEMTOP          0xa0000000

/*
 * Ecpu boot vectors and arguments.
 * See ecpuctx.h for details.
 */
#define PALOMEM_ECTX_NAME       "ectx"
#define PALOMEM_ECTX_BASE       0x00000000
#define PALOMEM_ECTX_SIZE       0x00002000

/*
 * Diags uses this page to pass data from the host diags app to diags fw.
 * Only used by the diags firmware image.
 */
#define PALOMEM_DIAG_NAME       "diag"
#define PALOMEM_DIAG_BASE       0x00002000
#define PALOMEM_DIAG_SIZE       0x00001000

/*
 * The memtun region contains the transmit/receive rings for
 * exchanging packets with the host through the memtun facility.
 * See sa/src/palo/fw/mcpu/memtun for details.
 */
#define PALOMEM_MEMTUN_NAME     "memtun"
#define PALOMEM_MEMTUN_BASE     0x00003000
#define PALOMEM_MEMTUN_SIZE     0x00040000

/*
 * Linux memory region for Linux kernel and user processes.
 * N.B. *** The Linux kernel load address load-$(CONFIG_NUOVA_PALO)
 * from mcpu/firmware/linux/arch/mips/Makefile must be within this
 * region ***.
 */
#define PALOMEM_LINUX_NAME      "linux"
#define PALOMEM_LINUX_BASE      0x00043000
#define PALOMEM_LINUX_SIZE      0x042bd000

/*
 * #if 0 //anadeepa - memtun relocate
  * #definePALOMEM_HWIO256_SIZE    0x01800000
   * #endif
    */
#define PALOMEM_HWIO256_SIZE    0x02800000

#ifdef CONFIG_BASE_SERENO
#define PALOMEM_FLOW_TBL_NAME   "flowtbl"
#define PALOMEM_FLOW_TBL_BASE   0x0a000000
#define PALOMEM_FLOW_TBL_SIZE   0x04000000
#endif

/*
 * Exchange Table will be shared between RoCE and FCoE
 * RoCE will use 32K exchanges and FCoE will use first
 * 24K exchanges. Hence increasing the exchange memory
 * to 16MB to accommodate all 64K exchanges.
 * The extra 10MB is grabbed from HWIO memory. It's
 * reduced from 55MB to 45MB now.
 */
#define PALOMEM_EXTBL_NAME      "extbl"
#define PALOMEM_EXTBL_BASE      0x04300000
#define PALOMEM_EXTBL_SIZE      0x01000000

/*
 * Part of the exchange table is used for cache in Palo but not in Sereno.
 * Need to know the amount that is cache in the palo_dfw.lds file.
 */
#define PALOMEM_EXTBL_CA_SIZE   0x00000000
/*
 * Exchange table param/stats size is 8MB each in Cruz.
 * Exchange Table is shared between RoCE and FCoE
 * RoCE will use 32K exchanges and FCoE will use first
 * 24K exchanges. Need 8MB each for param and stats to
 * accomodate all 64K exchanges.
 */
#define PALOMEM_EXTBL_PM_SIZE   0x00800000
#define PALOMEM_EXTBL_SS_SIZE   0x00800000

#define PALOMEM_ECPU1_NAME      "ecpu1"
#define PALOMEM_ECPU1_BASE      0x05300000
#define PALOMEM_ECPU1_SIZE      0x02000000

#define PALOMEM_ECPU2_NAME      "ecpu2"
#define PALOMEM_ECPU2_BASE      0x07300000
#define PALOMEM_ECPU2_SIZE      0x02000000

#define PALOMEM_ECPU3_NAME      "ecpu3"
#define PALOMEM_ECPU3_BASE      0x09300000
#define PALOMEM_ECPU3_SIZE      0x02000000

#define PALOMEM_ECPU4_NAME      "ecpu4"
#define PALOMEM_ECPU4_BASE      0x0b300000
#define PALOMEM_ECPU4_SIZE      0x02000000

/*
 * The hwio region is reserved for ecpu physical memory access.
 * See sa/src/common/libhwio/include/hwio_mipslinux_impl.h for details.
 */
#define PALOMEM_HWIO_NAME       "hwio"
#define PALOMEM_HWIO_BASE       0x0d300000
#define PALOMEM_HWIO_SIZE       0x01d00000

#define PALOMEM_PMPCIE_NAME     "pmpcie"
#define PALOMEM_PMPCIE_BASE     0x0f000000
#define PALOMEM_PMPCIE_SIZE     0x01000000

/*
 * The linuxhi region provides HIGHMEM RAM for Linux.
 */
#define PALOMEM_LINUXHI_NAME    "linuxhi"
#define PALOMEM_LINUXHI_BASE    0x90000000      // alias for 0x10000000
#define PALOMEM_LINUXHI_SIZE    0x08800000

/*
 * The fwdtab region supports the maximum multicast capacity for CE forwarding.
 * A forwarding table entry can use an external multicast list comprising
 * 2048 LIFs.  8 LIFs can be packed into 12 bytes, so a single multicast
 * list requires:
 *
 *      2048 * 12 / 8 = 3072 bytes.
 *
 * There are 8192 forwarding table entries, giving:
 *
 *      8192 * 3072 = 24MB
 */
#define PALOMEM_FWDTAB_NAME     "fwdtab"
#define PALOMEM_FWDTAB_BASE     0x98800000      // alias for 0x18800000
#define PALOMEM_FWDTAB_SIZE     0x01800000

/*
 * The viftbl region supports the maximum multicast capacity for VNTAG
 * forwarding.  The multicast lists are the same as the fwdtab, requiring
 * 3072 bytes / entry.
 *
 * There are 32768 entries in the viftbl, giving:
 *
 *      32768 * 3072 = 96MB
 */
#define PALOMEM_VIFTBL_NAME     "viftbl"
#define PALOMEM_VIFTBL_BASE     0x9a000000      // alias for 0x1a000000
#define PALOMEM_VIFTBL_SIZE     0x06000000

/*
 * The RDMA flow table region 64MB starting at the old genio boundary
 */
#define PALOMEM_RDMAFT_NAME    "rdmaft"
#define PALOMEM_RDMAFT_BASE    0x20000000
#define PALOMEM_RDMAFT_SIZE    0x04000000

/*
 * The USNIC flow table region 32MB starting at the old genio boundary
 */
#define PALOMEM_USNICFT_NAME   "usnicft"
#define PALOMEM_USNICFT_BASE   0x24000000
#define PALOMEM_USNICFT_SIZE   0x02000000

/*
 * The genio region can be used for general I/O.
 */
#define PALOMEM_GENIO_NAME      "genio"
#define PALOMEM_GENIO_BASE      0x26000000
#define PALOMEM_GENIO_SIZE      0x5a000000

#endif /* _PALO_PALOMEM_H_ */
